==> nagios-of-plugins-0.5.1.tar.gz: nagios-of-plugins-0.5.1/Nagios/Plugin.pm <==
#
# Common stuff for Nagios plugins
#

package Nagios::Plugin;

use strict;
require Exporter;
use File::Basename;
use Carp;
use Getopt::Long;
Getopt::Long::Configure('bundling');

use vars qw(@ISA @EXPORT $plugin $timeout $verbose $VERSION);

@ISA = qw(Exporter);
@EXPORT = qw($plugin $timeout $verbose parse_args exit exit_results);

$plugin = basename $0;
$timeout = 15;

my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);

# Nagios::Plugin version (not _plugin_ version - that's $version)
$VERSION = 0.01;

# ------------------------------------------------------------------------
# Private subroutines

sub print_revision 
{
  my ($version) = @_;
  $version .= "\n" unless substr($version,-1) eq "\n";
  print "$plugin $version";
  print qq(
This nagios plugin is free software, and comes with ABSOLUTELY NO WARRANTY. 
It may be used, redistributed and/or modified under the terms of the GNU 
General Public Licence (see http://www.fsf.org/licensing/licenses/gpl.txt).
);
}

sub print_help {
  my ($version, $usage, $help, $help_extra) = @_;
  print_revision($version);
  print "\n$usage";
  print $help;
  print $help_extra if $help_extra;
}

# ------------------------------------------------------------------------
# Public subroutines

# exit("CODE", "error string")
sub exit 
{
  my ($code, $errstr) = @_;
  $code ||= "UNKNOWN";
  die "invalid code '$code'" unless exists $ERRORS{$code};
  if ($errstr && $errstr ne '1') {
    $errstr .= "\n" unless substr($errstr,-1) eq "\n";
    my $short_name = uc $plugin;
    $short_name =~ s/^check_//i;
    print "$short_name $code - $errstr";
  }
  CORE::exit $ERRORS{$code};
}

# parse_args(version => $v, usage => $u, spec => \@s)
sub parse_args
{
  my %arg = @_;

  foreach (qw(usage version spec)) {
    croak "missing required argument '$_'" unless exists $arg{$_};
  }

  my $usage = sprintf $arg{usage}, $plugin;
  $usage .= "\n" unless substr($usage,-1) eq "\n";
  my $version = $arg{version};
  my $spec = $arg{spec};
  my $help_extra = $arg{help_extra};
  $help_extra .= "\n" if $help_extra && substr($help_extra,-1) ne "\n";
  croak "invalid spec argument - not array ref" unless ref $spec eq 'ARRAY';
  croak "odd number of spec elements" unless int(scalar @$spec % 2) == 0;

  # Build up options and help text from spec array
  my ($show_usage, $show_help, $show_version, @verbose);
  my @opt = (
    "usage|?"             => \$show_usage,
    "help|h"              => \$show_help,
    "version|V"           => \$show_version,
    "timeout|t=i"         => \$timeout, 
    "verbose|v"           => \@verbose,
  );
  my $help_tmpl = qq(Options:
 -h, --help
   Print detailed help screen
 -V, --version
   Print version information
%s
 -t, --timeout=INTEGER
   Seconds before plugin times out (default: $timeout)
 -v, --verbose
   Show details for command-line debugging 
);
  my $help = '';
  my @req = ();
  while (@$spec) {
    my $opt = shift @$spec;
    my $elt = shift @$spec;
    croak "spec element for option '$opt' is not array ref"
      unless ref $elt eq 'ARRAY';
    my ($var, $help_text, $req) = @$elt;
    push @opt, $opt, $var;
    $help_text = " $help_text" unless substr($help_text,0,1) eq ' ';
    $help_text .= "\n" unless substr($help_text,-1) eq "\n";
    $help .= $help_text;
    push @req, $var if $req;
  }
  chomp $help;
  $help = sprintf $help_tmpl, $help;

  my $result = GetOptions(@opt) || print($usage) && CORE::exit;

  # Handle standard options
  &exit("UNKNOWN", print_revision($version)) if $show_version;
  &exit("UNKNOWN", print($usage)) if $show_usage;
  &exit("UNKNOWN", print_help($version,$usage,$help,$help_extra)) if $show_help;

  # Check required arguments
  for (@req) {
    if (ref $_ eq 'SCALAR') {
      &exit("UNKNOWN", print($usage)) unless defined $$_;
    } elsif (ref $_ eq 'ARRAY') {
      &exit("UNKNOWN", print($usage)) unless @$_;
    } else {
      die "invalid required reference '$_' - expected scalar ref or array ref only";
    }
  }

  # Setup alarm handler (will handle any alarm($timeout) for plugin)
  $SIG{ALRM} = sub {
    &exit("UNKNOWN", "no response from $plugin (timeout, ${timeout}s)");
  };

  $verbose = scalar(@verbose);
}

# exit_results(%arg) 
#   returns CRITICAL if @{$arg{CRITICAL}}, WARNING if @{$arg{WARNING}}, else OK
#   uses $arg{results} for message if defined, else @{$arg{<STATUS>}}, 
#     where <STATUS> is the return code above
sub exit_results
{
  my (%arg) = @_;
  
  my %keys = map { $_ => 1 } qw(CRITICAL WARNING OK results);
  for (sort keys %arg) {
    croak "[Nagios::Plugin::exit_results] invalid argument $_" unless $keys{$_};
  }

  my $results = '';
  my $delim = ' : ';
  if ($arg{results}) {
    $results = ref $arg{results} eq 'ARRAY' ? 
      join($delim, @{$arg{results}}) : 
      $arg{results};
  }

  if ($arg{CRITICAL} && (ref $arg{CRITICAL} ne 'ARRAY' || @{$arg{CRITICAL}})) {
    &exit("CRITICAL", $results) if $results;
    &exit("CRITICAL", join($delim, @{$arg{CRITICAL}})) 
      if $arg{CRITICAL} && ref $arg{CRITICAL} eq 'ARRAY' && @{$arg{CRITICAL}};
    &exit("CRITICAL", $arg{CRITICAL}) if $arg{CRITICAL};
  }

  elsif ($arg{WARNING} && (ref $arg{WARNING} ne 'ARRAY' || @{$arg{WARNING}})) {
    &exit("WARNING", $results) if $results;
    &exit("WARNING", join($delim, @{$arg{WARNING}})) 
      if $arg{WARNING} && ref $arg{WARNING} eq 'ARRAY' && @{$arg{WARNING}};
    &exit("WARNING", $arg{WARNING}) if $arg{WARNING};
  }

  &exit("OK", $results) if $results;
  &exit("OK", join($delim, @{$arg{OK}})) 
    if $arg{OK} && ref $arg{OK} eq 'ARRAY' && @{$arg{OK}};
  &exit("OK", $arg{OK}) if $arg{OK};
  &exit("OK", "All okay");
}

# ------------------------------------------------------------------------

1;

__END__

=head1 NAME

Nagios::Plugin - Perl module for creating nagios plugins

=head1 SYNOPSIS

    # Nagios::Plugin exports $plugin, $timeout, and $verbose, as well as
    #   parse_args() and exit_results(), and overrides exit()
    use Nagios::Plugin;

    # parse_args - parse @ARGV for std args and args in @spec
    parse_args(
      version => 0.01,
      usage => 'usage: %s -w <warn> -c <crit>',
      spec => \@spec,
    );

    # exit($code, $msg) 
    #   where $code is qw(OK WARNING CRITICAL UNKNOWN DEPENDENT)
    exit("CRITICAL", "You're ugly and your mother dresses you funny");

    # exit_results - exit based on the given arrays
    exit_results(
      CRITICAL => \@crit,
      WARNING  => \@warn,
      OK       => $ok_message,
    );


=head1 DESCRIPTION

Nagios::Plugin is a perl module for simplifying the creation of 
nagios plugins, mainly by standardising some of the argument parsing
and handling stuff most plugins require.

Nagios::Plugin exports 3 variables - $plugin (the name of the plugin
i.e. basename($0)); $timeout (the number of seconds before the plugin
times out); and $verbose (the number of -v arguments). It also exports 
two subroutines: parse_args(), for parsing @ARGV for std and supplied 
args, and exit_results(), for checking a set of arrays and exiting 
appropriately, and overrides core exit() with a version that returns 
a standard Nagios return status plus a message.

=head2 parse_args

The parse_args subroutine provides the core Nagios::Plugin functionality, 
and should be called early in your plugin. It uses a named argument
syntax, and currently takes three arguments: the version number ('version'); 
a short usage message ('usage', newlines okay, %s will be substituted with 
$plugin); and an argument spec for any additional arguments your plugin 
will accept.

Nagios::Plugin provides standard argument handling for the following 
arguments:

=over 4

=item --usage | -?

Print a short usage message for the plugin.

=item --help | -h

Print a longer help message, include the version number, the usage
message, and help text for the individual arguments.

=item --version | -V

Print the plugin version number and licence information (currently
hardcoded).

=item --timeout | -t

Number of seconds for plugin timeout (if the plugin implements one);
default: 15.

=item --verbose | -v

Turn on verbose debug output (may be repeated - the exported $verbose 
variable is set to the number of -v arguments parsed).

=back

To define additional arguments for your plugin, you fill out an
argument specifier, which is just an arrayref of argument definitions.
Each definition is basically an extended Getopt::Long definition - 
instead of the two-item tuple $option_defn => $variable used in
Getopt::Long, Nagios::Plugin uses a three- or four-item tuple which 
also includes help text for argument, and optionally an additional
mandatoriness flag - the syntax used is:

  $option_defn => [ $variable, $help_text, $required_flag ]

e.g.

  parse_args(
    version => 0.01,
    usage => 'usage: %s -w <warn> -c <crit>',
    spec => [
      "warning|w=s" => [
        \$warning, 
        q(-w, --warning=INTEGER\n   Exit with WARNING status if less than INTEGER foobars are free),
        'REQUIRED',
      ],
      "critical|c=s" => [
        \$critical, 
        q(-c, --critical=INTEGER\n   Exit with CRITICAL status if less than INTEGER foobars are free),
        'REQUIRED',
      ],
    ],
  );

=head2 exit_results

exit_results exits from the plugin after examining a supplied set of 
arrays. Syntax is:

  exit_results(
    CRITICAL => \@crit,
    WARNING  => \@warn,
    OK       => $ok_message,    # or \@ok_messages
  );

exit_results returns 'CRITICAL' if the @crit array is non-empty;
'WARNING' if the @warn array is non-empty; and otherwise 'OK'. The
text returned is typically the joined contents of @crit or @warn or
@ok_messages (or $ok_message).

Sometimes on error or warning you want to return more than just the 
error cases in the returned text. You can do this by passing a
'results' parameter containing the string or strings you want to use
for all status codes e.g.

  # Use the given $results string on CRITICAL, WARNING, and OK
  exit_results(
    CRITICAL => \@crit,
    WARNING  => \@warn,
    results => $results         # or \@results
  );


=head1 AUTHOR

Gavin Carr <gavin@openfusion.com.au>


=head1 LICENCE

Copyright 2005 Gavin Carr. All Rights Reserved.

This module is free software. It may be used, redistributed
and/or modified under either the terms of the Perl Artistic 
License (see http://www.perl.com/perl/misc/Artistic.html)
or the GNU General Public Licence (see 
http://www.fsf.org/licensing/licenses/gpl.txt).

=cut

# arch-tag: 1495e893-2a66-4e61-a8eb-8bfa401b2a4f
# vim:ft=perl:ai:sw=2

==> nagios-of-plugins-0.6.tar.gz: nagios-of-plugins-0.6/Nagios/Plugin.pm <==
#
# Common stuff for Nagios plugins
#

package Nagios::Plugin;

use strict;
require Exporter;
use File::Basename;
use Config::Tiny;
use Carp;
use Getopt::Long;
Getopt::Long::Configure('bundling');

use vars qw(@ISA @EXPORT $PLUGIN $TIMEOUT $CONFIG $VERBOSE $VERSION);

@ISA = qw(Exporter);
@EXPORT = qw($PLUGIN $TIMEOUT $CONFIG $VERBOSE parse_args exit exit_results);

$PLUGIN = basename $0;
$TIMEOUT = 15;
$CONFIG = load_config();

my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);

# Nagios::Plugin version (not _plugin_ version)
$VERSION = 0.02;

# ------------------------------------------------------------------------
# Private subroutines

sub print_revision 
{
  my ($version) = @_;
  $version .= "\n" unless substr($version,-1) eq "\n";
  print "$PLUGIN $version";
  print qq(
This nagios plugin is free software, and comes with ABSOLUTELY NO WARRANTY. 
It may be used, redistributed and/or modified under the terms of the GNU 
General Public Licence (see http://www.fsf.org/licensing/licenses/gpl.txt).
);
}

sub print_help {
  my ($version, $usage, $help, $help_extra) = @_;
  print_revision($version);
  print "\n$usage";
  print $help;
  print $help_extra if $help_extra;
}

sub load_config {
  my $config_file = '/etc/nagios/plugins.cfg';
  return {} unless -f $config_file;
  my $ct = Config::Tiny->read($config_file);
  return { %{$ct->{_} || {}}, %{$ct->{$PLUGIN} || {}} } if ref $ct;
  return {};
}

# ------------------------------------------------------------------------
# Public subroutines

# exit("CODE", "error string")
sub exit 
{
  my ($code, $errstr) = @_;
  $code ||= "UNKNOWN";
  die "invalid code '$code'" unless exists $ERRORS{$code};
  if ($errstr && $errstr ne '1') {
    $errstr .= "\n" unless substr($errstr,-1) eq "\n";
    my $short_name = uc $PLUGIN;
    $short_name =~ s/^check_//i;
    print "$short_name $code - $errstr";
  }
  CORE::exit $ERRORS{$code};
}

# parse_args(version => $v, usage => $u, spec => \@s)
sub parse_args
{
  my %arg = @_;

  foreach (qw(usage version spec)) {
    croak "missing required argument '$_'" unless exists $arg{$_};
  }

  my $usage = sprintf $arg{usage}, $PLUGIN;
  $usage .= "\n" unless substr($usage,-1) eq "\n";
  my $version = $arg{version};
  my $spec = $arg{spec};
  my $help_extra = $arg{help_extra};
  $help_extra .= "\n" if $help_extra && substr($help_extra,-1) ne "\n";
  croak "invalid spec argument - not array ref" unless ref $spec eq 'ARRAY';
  croak "odd number of spec elements" unless int(scalar @$spec % 2) == 0;

  # Build up options and help text from spec array
  my ($show_usage, $show_help, $show_version, @verbose);
  my @opt = (
    "usage|?"             => \$show_usage,
    "help|h"              => \$show_help,
    "version|V"           => \$show_version,
    "timeout|t=i"         => \$TIMEOUT, 
    "verbose|v"           => \@verbose,
  );
  my $help_tmpl = qq(Options:
 -h, --help
   Print detailed help screen
 -V, --version
   Print version information
%s
 -t, --timeout=INTEGER
   Seconds before plugin times out (default: $TIMEOUT)
 -v, --verbose
   Show details for command-line debugging 
);
  my $help = '';
  my @req = ();
  while (@$spec) {
    my $opt = shift @$spec;
    my $elt = shift @$spec;
    croak "spec element for option '$opt' is not array ref"
      unless ref $elt eq 'ARRAY';
    my ($var, $help_text, $req) = @$elt;
    push @opt, $opt, $var;
    $help_text = " $help_text" unless substr($help_text,0,1) eq ' ';
    $help_text .= "\n" unless substr($help_text,-1) eq "\n";
    $help .= $help_text;
    push @req, $var if $req;
  }
  chomp $help;
  $help = sprintf $help_tmpl, $help;

  my $result = GetOptions(@opt) || print($usage) && CORE::exit;

  # Handle standard options
  &exit("UNKNOWN", print_revision($version)) if $show_version;
  &exit("UNKNOWN", print($usage)) if $show_usage;
  &exit("UNKNOWN", print_help($version,$usage,$help,$help_extra)) if $show_help;

  # Check required arguments
  for (@req) {
    if (ref $_ eq 'SCALAR') {
      &exit("UNKNOWN", print($usage)) unless defined $$_;
    } elsif (ref $_ eq 'ARRAY') {
      &exit("UNKNOWN", print($usage)) unless @$_;
    } else {
      die "invalid required reference '$_' - expected scalar ref or array ref only";
    }
  }

  # Setup alarm handler (will handle any alarm($TIMEOUT) for plugin)
  $SIG{ALRM} = sub {
    &exit("UNKNOWN", "no response from $PLUGIN (timeout, ${TIMEOUT}s)");
  };

  $VERBOSE = scalar(@verbose);
}

# exit_results(%arg) 
#   returns CRITICAL if @{$arg{CRITICAL}}, WARNING if @{$arg{WARNING}}, else OK
#   uses $arg{results} for message if defined, else @{$arg{<STATUS>}}, 
#     where <STATUS> is the return code above
sub exit_results
{
  my (%arg) = @_;
  
  my %keys = map { $_ => 1 } qw(CRITICAL WARNING OK results);
  for (sort keys %arg) {
    croak "[Nagios::Plugin::exit_results] invalid argument $_" unless $keys{$_};
  }

  my $results = '';
  my $delim = ' : ';
  if ($arg{results}) {
    $results = ref $arg{results} eq 'ARRAY' ? 
      join($delim, @{$arg{results}}) : 
      $arg{results};
  }

  if ($arg{CRITICAL} && (ref $arg{CRITICAL} ne 'ARRAY' || @{$arg{CRITICAL}})) {
    &exit("CRITICAL", $results) if $results;
    &exit("CRITICAL", join($delim, @{$arg{CRITICAL}})) 
      if $arg{CRITICAL} && ref $arg{CRITICAL} eq 'ARRAY' && @{$arg{CRITICAL}};
    &exit("CRITICAL", $arg{CRITICAL}) if $arg{CRITICAL};
  }

  elsif ($arg{WARNING} && (ref $arg{WARNING} ne 'ARRAY' || @{$arg{WARNING}})) {
    &exit("WARNING", $results) if $results;
    &exit("WARNING", join($delim, @{$arg{WARNING}})) 
      if $arg{WARNING} && ref $arg{WARNING} eq 'ARRAY' && @{$arg{WARNING}};
    &exit("WARNING", $arg{WARNING}) if $arg{WARNING};
  }

  &exit("OK", $results) if $results;
  &exit("OK", join($delim, @{$arg{OK}})) 
    if $arg{OK} && ref $arg{OK} eq 'ARRAY' && @{$arg{OK}};
  &exit("OK", $arg{OK}) if $arg{OK};
  &exit("OK", "All okay");
}

# ------------------------------------------------------------------------

1;

__END__

=head1 NAME

Nagios::Plugin - Perl module for creating nagios plugins

=head1 SYNOPSIS

    # Nagios::Plugin exports $PLUGIN, $CONFIG, $TIMEOUT, and $VERBOSE, 
    #   and two subroutines: parse_args() and exit_results(), and 
    #   overrides exit()
    use Nagios::Plugin;

    # parse_args - parse @ARGV for std args and args in @spec
    parse_args(
      version => 0.01,
      usage => 'usage: %s -w <warn> -c <crit>',
      spec => \@spec,
    );

    # exit($code, $msg) 
    #   where $code is qw(OK WARNING CRITICAL UNKNOWN DEPENDENT)
    exit("CRITICAL", "You're ugly and your mother dresses you funny");

    # exit_results - exit based on the given arrays
    exit_results(
      CRITICAL => \@crit,
      WARNING  => \@warn,
      OK       => $ok_message,
    );


=head1 DESCRIPTION

Nagios::Plugin is a perl module for simplifying the creation of 
nagios plugins, mainly by standardising some of the argument parsing
and handling stuff most plugins require.

Nagios::Plugin exports the following four variables:

=over 4

=item $PLUGIN 

The name of the plugin i.e. basename($0).
  
=item $CONFIG

A hashref of configuration items defined in the nagios plugins.cfg file for 
the current plugin.

=item $TIMEOUT 

The number of seconds before the plugin times out, set via the -t argument.

=item $VERBOSE 

The number of -v arguments to the plugin.
    
=back

Nagios::Plugin also exports two subroutines: parse_args(), for parsing @ARGV 
for std and supplied args; and exit_results(), for checking a set of arrays 
and exiting appropriately, and overrides core exit() with a version that 
returns a standard Nagios return status plus a message.

=head2 parse_args

The parse_args subroutine provides the core Nagios::Plugin functionality, 
and should be called early in your plugin. It uses a named argument
syntax, and currently takes three arguments: the version number ('version'); 
a short usage message ('usage', newlines okay, %s will be substituted with 
$PLUGIN); and an argument spec for any additional arguments your plugin 
will accept.

Nagios::Plugin provides standard argument handling for the following 
arguments:

=over 4

=item --usage | -?

Print a short usage message for the plugin.

=item --help | -h

Print a longer help message, include the version number, the usage
message, and help text for the individual arguments.

=item --version | -V

Print the plugin version number and licence information (currently
hardcoded).

=item --timeout | -t

Number of seconds for plugin timeout (if the plugin implements one);
default: 15.

=item --verbose | -v

Turn on verbose debug output (may be repeated - the exported $VERBOSE 
variable is set to the number of -v arguments parsed).

=back

To define additional arguments for your plugin, you fill out an
argument specifier, which is just an arrayref of argument definitions.
Each definition is basically an extended Getopt::Long definition - 
instead of the two-item tuple $option_defn => $variable used in
Getopt::Long, Nagios::Plugin uses a three- or four-item tuple which 
also includes help text for argument, and optionally an additional
mandatoriness flag - the syntax used is:

  $option_defn => [ $variable, $help_text, $required_flag ]

e.g.

  parse_args(
    version => 0.01,
    usage => 'usage: %s -w <warn> -c <crit>',
    spec => [
      "warning|w=s" => [
        \$warning, 
        q(-w, --warning=INTEGER\n   Exit with WARNING status if less than INTEGER foobars are free),
        'REQUIRED',
      ],
      "critical|c=s" => [
        \$critical, 
        q(-c, --critical=INTEGER\n   Exit with CRITICAL status if less than INTEGER foobars are free),
        'REQUIRED',
      ],
    ],
  );

=head2 exit_results

exit_results exits from the plugin after examining a supplied set of 
arrays. Syntax is:

  exit_results(
    CRITICAL => \@crit,
    WARNING  => \@warn,
    OK       => $ok_message,    # or \@ok_messages
  );

exit_results returns 'CRITICAL' if the @crit array is non-empty;
'WARNING' if the @warn array is non-empty; and otherwise 'OK'. The
text returned is typically the joined contents of @crit or @warn or
@ok_messages (or $ok_message).

Sometimes on error or warning you want to return more than just the 
error cases in the returned text. You can do this by passing a
'results' parameter containing the string or strings you want to use
for all status codes e.g.

  # Use the given $results string on CRITICAL, WARNING, and OK
  exit_results(
    CRITICAL => \@crit,
    WARNING  => \@warn,
    results => $results         # or \@results
  );


=head1 AUTHOR

Gavin Carr <gavin@openfusion.com.au>


=head1 LICENCE

Copyright 2005 Gavin Carr. All Rights Reserved.

This module is free software. It may be used, redistributed
and/or modified under either the terms of the Perl Artistic 
License (see http://www.perl.com/perl/misc/Artistic.html)
or the GNU General Public Licence (see 
http://www.fsf.org/licensing/licenses/gpl.txt).

=cut

# arch-tag: 1495e893-2a66-4e61-a8eb-8bfa401b2a4f
# vim:ft=perl:ai:sw=2
